<?php
/*
Plugin Name: Ning Publisher
Plugin URI: http://www.ning.com/
Description: Publish blog posts to your Ning Network.
Version: 1.0.3
Author: Ning, Inc
Author URI: http://www.ning.com/
License:

NING DOES NOT AND CANNOT WARRANT THE PERFORMANCE OR RESULTS YOU MAY OBTAIN BY
USING THE TECHNOLOGY. NING MAKES NO WARRANTIES, EXPRESS OR IMPLIED, AS TO
NON-INFRINGEMENT OF THIRD PARTY RIGHTS, MERCHANTABILITY, OR FITNESS FOR ANY
PARTICULAR PURPOSE. IN NO EVENT WILL NING OR ITS SUPPLIERS BE LIABLE TO YOU
FOR ANY CONSEQUENTIAL, INCIDENTAL, OR SPECIAL DAMAGES, INCLUDING ANY LOST
PROFITS OR LOST SAVINGS, EVEN IF A NING REPRESENTATIVE HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES, OR FOR ANY CLAIM BY ANY THIRD PARTY.

A portion of the code supplied in this plugin is subject to the BSD license as
follows:

<OWNER> = Ning Inc.
<ORGANIZATION> = Ning Inc.
<YEAR> = 2010

Copyright (c) 2010, Ning Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

* Neither the name of Ning Inc. nor the names of its contributors may be used
  to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/**
 * Known issues:
 * - As long as the upload directory is publicly visible, the images should
 *   work
 */

require_once("NingApi.php");

$plugin_dir = basename(dirname(__FILE__));
load_plugin_textdomain('ning-publisher', 'wp-content/plugins/' . $plugin_dir,
    $plugin_dir);


// -----------------------------------------------------------------------------
// Settings API
// -----------------------------------------------------------------------------

/**
 * Setup the settings
 */
function ning_pub_admin_init () {
	// Create a new section
	$section_id = 'ning_pub_settings_section';
	$section_title = 'General';
	$section_callback = 'ning_pub_section_callback';
	$section_page = 'ning_pub_default_page';
	add_settings_section($section_id, $section_title, $section_callback,
	    $section_page);


	$setting_group = 'ning_pub_settings_group';
	$setting_page = 'ning_pub_default_page';
	$setting_section = $section_id;

	// Network url setting
	$setting_id = 'ning_pub_network_url';
	$setting_title = 'Network Subdomain';
	$setting_callback = 'ning_pub_network_url_callback';
	$setting_args = array();
	register_setting($setting_group, $setting_id);
	add_settings_field($setting_id, $setting_title, $setting_callback,
	    $setting_page, $setting_section, $setting_args);

	// Consumer key setting
	$setting_id = 'ning_pub_consumer_key';
	$setting_title = 'Consumer Key';
	$setting_callback = 'ning_pub_consumer_key_callback';
	$setting_args = array();
	register_setting($setting_group, $setting_id);
	add_settings_field($setting_id, $setting_title, $setting_callback,
	    $setting_page, $setting_section, $setting_args);

	// Consumer secret setting
	$setting_id = 'ning_pub_consumer_secret';
	$setting_title = 'Consumer Secret';
	$setting_callback = 'ning_pub_consumer_secret_callback';
	$setting_args = array();
	register_setting($setting_group, $setting_id);
	add_settings_field($setting_id, $setting_title, $setting_callback,
	    $setting_page, $setting_section, $setting_args);

}
add_action('admin_init', 'ning_pub_admin_init');

/**
 * Callback for the section header
 */
function ning_pub_section_callback() {
	return;
}

/**
 * Callback for the network URL settings HTML
 */
function ning_pub_network_url_callback() {
    $value = "";

	$domain = get_option('ning_pub_network_url');
    if ($domain) {
        $value = "value='$domain'";
    }
?>
<input id='ning-pub-domain-setting' name='ning_pub_network_url' type='text' <?php echo $value ?> size='40' />
<span class='description'>The subdomain name for your Ning Network (ie. 'examplenetwork' in examplenetwork.ning.com)</span>
<?php
}

/**
 * Callback for the Consumer Key settings HTML
 */
function ning_pub_consumer_key_callback() {
    $value = "";

	$domain = get_option('ning_pub_consumer_key');
    if ($domain) {
        $value = "value='$domain'";
    }
?>
<input id='ning-pub-consumer-key' name='ning_pub_consumer_key' type='text' <?php echo $value ?> size='40' />
<span class='description'>The consumer key given by Ning</span>
<?php
}

/**
 * Callback for the Consumer Key settings HTML
 */
function ning_pub_consumer_secret_callback() {
    $value = "";

	$domain = get_option('ning_pub_consumer_secret');
    if ($domain) {
        $value = "value='$domain'";
    }
?>
<input id='ning-pub-consumer-secret' name='ning_pub_consumer_secret' type='text' <?php echo $value ?> size='40' />
<span class='description'>The consumer secret give by Ning</span>
<?php
}


// -----------------------------------------------------------------------------
// Administration Menu
// -----------------------------------------------------------------------------

/**
 * Builds the admin option menu
 *
 */
function ning_pub_create_menu() {
	$page_title = 'Ning Publisher';
	$menu_title = $page_title;
	$capability = 'manage_options';
	$menu_slug = 'ning-publisher';
	$function = 'ning_pub_settings';

	add_options_page($page_title, $menu_title, $capability, $menu_slug,
	    $function);
}
/* Registers the menu building code */
add_action('admin_menu', 'ning_pub_create_menu');

/* Settings page building code */
function ning_pub_settings () {

    if (!current_user_can('manage_options'))  {
        wp_die(__('You do not have sufficient permissions to access this page.'));
    }
?>
<div class="wrap">
<h2>Ning Publisher</h2>
<form method="post" action="options.php">
    <?php settings_fields('ning_pub_settings_group'); ?>
    <?php do_settings_sections('ning_pub_default_page'); ?>

    <p class="submit">
    <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
    </p>
</form>
</div>
<?php
}



// -----------------------------------------------------------------------------
// Profile options
// -----------------------------------------------------------------------------

// http://justintadlock.com/archives/2009/09/10/adding-and-using-custom-user-profile-fields

/**
 * Display the token credential profile questions
 */
function ning_pub_show_profile_questions($user) {
    $params = array(
        "subdomain" => get_option('ning_pub_network_url'),
        "consumer_key" => get_option('ning_pub_consumer_key'),
        "consumer_secret" => get_option('ning_pub_consumer_secret')
    );
    $query = http_build_query($params);

    $key_page_path = WP_PLUGIN_URL . '/' . str_replace(basename( __FILE__),
        "",plugin_basename(__FILE__)) . 'ning-keys.php?' . $query;
?>

	<h3>Ning Publisher</h3>

	<table class="form-table">

		<tr>
			<th><label for="ning_pub_token">Access Token</label></th>

			<td>
				<input type="text" name="ning_pub_token" id="ning_pub_token" value="<?php echo esc_attr(get_the_author_meta('ning_pub_token', $user->ID)); ?>" class="regular-text" /><br />
			</td>
		</tr>

		<tr>
			<th><label for="ning_pub_secret">Access Token Secret</label></th>

			<td>
				<input type="text" name="ning_pub_secret" id="ning_pub_secret" value="<?php echo esc_attr(get_the_author_meta('ning_pub_secret', $user->ID)); ?>" class="regular-text" /><br />
			</td>
		</tr>
	</table>
    <p>To generate an access token visit <a target="_blank" href="<?php print $key_page_path ?>">Ning Key</a>.</p>
<?php }
add_action('show_user_profile', 'ning_pub_show_profile_questions'); // "Profile" - user
add_action('edit_user_profile', 'ning_pub_show_profile_questions'); // "Edit User" - admin

/**
 * Save the token credential profile questions
 */
function ning_pub_save_profile_questions($user_id) {
	if (!current_user_can('edit_user', $user_id))
		return false;

    $token = $_POST['ning_pub_token'];
	update_user_meta($user_id, 'ning_pub_token', $token);

    $secret = $_POST['ning_pub_secret'];
	update_user_meta($user_id, 'ning_pub_secret', $secret);
}
add_action('personal_options_update', 'ning_pub_save_profile_questions');
add_action('edit_user_profile_update', 'ning_pub_save_profile_questions');



// -----------------------------------------------------------------------------
// Publish functions
// -----------------------------------------------------------------------------

function ning_pub_make_request($post_id, $post, $method) {
    $subdomain = get_option('ning_pub_network_url', false);

    $access_token = get_user_meta($post->post_author, 'ning_pub_token', true);
    $access_token_secret = get_user_meta($post->post_author,
        'ning_pub_secret', true);

    $consumer_key = get_option('ning_pub_consumer_key');
    $consumer_secret = get_option('ning_pub_consumer_secret');

    if (!$subdomain ||
        empty($access_token) ||
        empty($access_token_secret) ||
        empty($consumer_key) ||
        empty($consumer_secret)) {
        // Missing credentials, fail silently
        return;
    }

    $ningApi = new NingApi($subdomain, $consumer_key, $consumer_secret,
        $access_token, $access_token_secret);

    // Process the post title and body through the normal filters
	$filtered_post_title = apply_filters('the_title', $post->post_title,
	    $post_id );
	$filtered_post_content = apply_filters('the_content', $post->post_content,
	    $post_id);
    $post_fields = array(
        "title" => $filtered_post_title,
        "description" => $filtered_post_content
    );

    if ($method == "PUT") {
        $ning_post_id = get_post_meta($post_id, "ning_pub_post_id", TRUE);
        if (!$ning_post_id) {
            wp_die("Missing ning_pub_post_id");
        }
        $post_fields["id"] = $ning_post_id;

        $result = $ningApi->put("BlogPost", $post_fields);
    } else if ($method == "POST") {
        $result = $ningApi->post("BlogPost", $post_fields);
        add_post_meta($post_id, "ning_pub_post_id", $result["id"], TRUE);
    } else {
        wp_die("Invalid method: " + $method);
    }
}

/**
 * Inserts or updates a published post on the Ning Network
 *
 */
function ning_pub_update_post($post_id, $post) {
    if (property_exists($post, "post_status") &&
        $post->post_status == "publish") {

        $ning_post_id = get_post_meta($post_id, "ning_pub_post_id", TRUE);
        if ($ning_post_id) {
            ning_pub_make_request($post_id, $post, "PUT");
            // wp_die("Update Post");
        } else {
            ning_pub_make_request($post_id, $post, "POST");
            // wp_die("Create Post");
        }
    }
}
add_action('save_post', 'ning_pub_update_post', 10, 2);
